--[[
    RWN_ArchDistance 
    Copyright (C) 2018  RWN

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
]]--
local HBD = LibStub("HereBeDragons-2.0")
local PI = PI;
local TWOPI = PI * 2.0;
local cos = math.cos;
local PULSE_SEC	= 1.0 / (2.0*1.0);	

function DistanceIndicatorFrame_UpdateText(self, distance)

		text = math.ceil(distance)

	self:SetText(text)
end

function DistanceIndicatorFrame_UpdateColor(self, distance)
	local greenMin, greenMax = 0, 40
	local yellowMin, yellowMax = greenMax, 80
	local redMin, redMax = yellowMax, 500

	if distance >= greenMin and distance <= greenMax then
		DistanceIndicatorFrame_SetColorGreen()
	elseif distance >= yellowMin and distance <= yellowMax then
		DistanceIndicatorFrame_SetColorYellow()
	elseif distance >= redMin and distance <= redMax then
		DistanceIndicatorFrame_SetColorRed()
	end
	
end

function DistanceIndicatorFrame_SetColorGreen()
	DistanceGreenTex:Show()
	DistanceYellowTex:Hide()
	DistanceRedTex:Hide()
	DistanceGreyTex:Hide()
end

function DistanceIndicatorFrame_SetColorYellow()
	DistanceGreenTex:Hide()
	DistanceYellowTex:Show()
	DistanceRedTex:Hide()
	DistanceGreyTex:Hide()
end

function DistanceIndicatorFrame_SetColorRed()
	DistanceGreenTex:Hide()
	DistanceYellowTex:Hide()
	DistanceRedTex:Show()
	DistanceGreyTex:Hide()
end

function DistanceIndicatorFrame_SetColorGrey()
	DistanceGreenTex:Hide()
	DistanceYellowTex:Hide()
	DistanceRedTex:Hide()
	DistanceGreyTex:Show()
end


function DistanceIndicatorFrame_OnLoad(self)

	self:RegisterEvent("ARCHAEOLOGY_SURVEY_CAST")
	self:RegisterEvent("ARCHAEOLOGY_FIND_COMPLETE")
	self:RegisterEvent("PLAYER_STARTED_MOVING")
	self:RegisterEvent("PLAYER_STOPPED_MOVING")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:EnableMouse(true)
	self:RegisterForClicks("AnyUp")
	self:SetMovable(true)
	self:RegisterForDrag("LeftButton")
	self:SetAttribute("type1", "spell")
	self:SetAttribute("spell", select(1, GetSpellInfo(80451)))
	self:SetAttribute("unit", "player")

	self:GetFontString():SetDrawLayer("OVERLAY");

	
	self:SetFrameLevel(self:GetFrameLevel() + 2);

	self.lastSurveyPositionX, self.lastSurveyPositionY, self.lastSurveyPositionInstanceID = 0, 0, 0
	self.active = false
	self.moving = false
	self.inCombat = false
	self.standingOnIt = false
	self.flashTimer = 0
	
	DistanceIndicatorFrame_SetColorGreen()
end

function DistanceIndicatorFrame_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT");
end

function DistanceIndicatorFrame_OnEvent(self, event, arg1, arg2, arg3, ...)
		if ( event == "ARCHAEOLOGY_SURVEY_CAST") then
		
			self.active = true
			self.lastSurveyPositionX, self.lastSurveyPositionY, self.lastSurveyPositionInstanceID = HBD:GetPlayerWorldPosition()
			DistanceIndicatorFrame_UpdateText(self, 0)
			DistanceIndicatorFrame_OnUpdate(self);
			DistanceIndicatorFrame_SetColorGreen()
			self:Show();
		end
		
		if (event == "PLAYER_STARTED_MOVING" and self.active == true) then
			self.moving = true

		end	
		--[[
		This is preventing a call to the protected :Show() Function in combat
		Thanks to Ambiguouskid on curseforge for this bug report!
		]]--
		if (event == "PLAYER_REGEN_DISABLED") then
			self.inCombat = true
			self.active = false
			self.standingOnIt = false
			self.flashTimer = 0.0
			self:Hide();

		end	
		
		if (event == "PLAYER_REGEN_ENABLED") then
			self.inCombat = false

		end		
		if (event == "UNIT_SPELLCAST_SUCCEEDED" and self.active == true) then
			if (arg3 == 210837) then
				self.standingOnIt = true
			end
		end
		
		if (event == "PLAYER_STOPPED_MOVING") then
			self.moving = false
			if(self:IsShown() == false and CanScanResearchSite() and self.inCombat == false) then
				DistanceIndicatorFrame_SetColorGrey()
				DistanceIndicatorFrame_UpdateText(self, 0)
				self:Show();
			end

		end
		
		if (event == "ARCHAEOLOGY_FIND_COMPLETE") then
			self.active = false
			self.standingOnIt = false
			self.flashTimer = 0.0
			if(CanScanResearchSite()) then
				DistanceIndicatorFrame_SetColorGrey()
				DistanceIndicatorFrame_UpdateText(self, 0)
				
			else
				self:Hide();
			end
		end
		
		if (event == "UNIT_AURA" and self.active == true and arg1 == "player") then
			local y,n,s,_
			for i=1,40
			do
				name,_,_,_,_,_,_,_,_,id=UnitAura("player",i)
				if(id) then
					if(id == 210837) then
						y = 1
					end
					end
			end
			if y~=1 then
				self.standingOnIt=false
			end
		end
end

function DistanceIndicatorFrame_OnUpdate(self, elapsed)
	if (self.moving == true and self.active == true) then
		local playerPositionX, playerPositionY, playerPositionInstanceID = HBD:GetPlayerWorldPosition()
		local dist, deltaX, deltaY = HBD:GetWorldDistance(playerPositionInstanceID, playerPositionX, playerPositionY, self.lastSurveyPositionX, self.lastSurveyPositionY)
		DistanceIndicatorFrame_UpdateText(self, dist)
		DistanceIndicatorFrame_UpdateColor(self, dist)
	end
	
	if(GameTooltip:IsOwned(self)) then
		GameTooltip:ClearLines();
		GameTooltip:AddLine("RWN_ArchDistance");
		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("Zurückgelegte Strecke: ", self:GetText() .. " Meter")
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("Linksklick: ", select(1, GetSpellInfo(80451)))
		GameTooltip:AddDoubleLine("Rechtsklick: ", "Archäologie UI")
		GameTooltip:AddDoubleLine("Zum Verschieben: ", "gedrückt halten")

		
		GameTooltip:Show();
	end
	-- Flashing stuff
	if ( elapsed and self.standingOnIt ) then
	if (self.flashTimer == 0.0) then
		PlaySound(SOUNDKIT.UI_GROUP_FINDER_RECEIVE_APPLICATION);
	end
		local flashIndex = TWOPI * self.flashTimer * PULSE_SEC;
		local flashValue = max(0.0, 0.5 + 0.5*cos(flashIndex));
		if ( flashIndex >= TWOPI ) then
			self.flashTimer = 0.0;
		else
			self.flashTimer = self.flashTimer + elapsed;
		end

		DistanceIndicatorStandingOnItPulseTexture:SetAlpha(flashValue);
	else 
		DistanceIndicatorStandingOnItPulseTexture:SetAlpha(0);
	end
	
	
	if(CanScanResearchSite() == false ) then
		self:Hide()
	end


end

function DistanceIndicatorFrame_PostClick(self, button)
	if (button == "RightButton") then
		if ArchaeologyFrame:IsShown() then
			--Archaeology window is open
			HideUIPanel(ArchaeologyFrame)
		else
			--Archaeology window is closed
			ShowUIPanel(ArchaeologyFrame)
		end
	end
end

function DistanceIndicatorFrame_OnDragStart(self)
	self:StartMoving()
end

function DistanceIndicatorFrame_OnDragStop(self)
	self:StopMovingOrSizing()
end
